@@ -47,8 +47,14 @@ class ClickReviewSecurity(ClickReview):
         ClickReview.__init__(self, fn, "security", peer_hooks=peer_hooks,
                              overrides=overrides)
 
-        local_copy = os.path.join(os.path.dirname(__file__),
-                                  '../data/apparmor-easyprof-ubuntu.json')
+        # If local_copy is None, then this will check the server to see if
+        # we are up to date. However, if we are working within the development
+        # tree, use it unconditionally.
+        local_copy = None
+        branch_fn = os.path.join(os.path.dirname(__file__),
+                                 '../data/apparmor-easyprof-ubuntu.json')
+        if os.path.exists(branch_fn):
+            local_copy = branch_fn
         p = apparmor_policy.ApparmorPolicy(local_copy)
         self.aa_policy = p.policy
 
@@ -1286,3 +1292,31 @@ class ClickReviewSecurity(ClickReview):
                     continue
 
         # TODO: error if not 'common'
+
+    def check_template_online_accounts_provider(self):
+        '''Check template for online accounts account-provider'''
+        for app in sorted(self.security_apps):
+            (f, m) = self._get_security_manifest(app)
+            t = 'info'
+            n = 'template_account_provider(%s)' % f
+            s = "OK"
+            if 'account-provider' not in self.manifest['hooks'][app]:
+                continue
+            if 'template' not in m or m['template'] != "ubuntu-account-plugin":
+                t = 'error'
+                s = "template is not 'ubuntu-account-plugin'"
+            self._add_result(t, n, s)
+
+    def check_template_online_accounts_qml_plugin(self):
+        '''Check template for online accounts account-qml-plugin'''
+        for app in sorted(self.security_apps):
+            (f, m) = self._get_security_manifest(app)
+            t = 'info'
+            n = 'template_account_qml_plugin(%s)' % f
+            s = "OK"
+            if 'account-qml-plugin' not in self.manifest['hooks'][app]:
+                continue
+            if 'template' not in m or m['template'] != "ubuntu-account-plugin":
+                t = 'error'
+                s = "template is not 'ubuntu-account-plugin'"
+            self._add_result(t, n, s)
