@@ -40,15 +40,24 @@ class ClickReviewSystemd(ClickReview):
         # - stop
         # - poststop
         # - stop-timeout
-        # - TODO: caps
+        # - caps (checked in in cr_security.py)
+        # - security-template (checked in in cr_security.py)
+        # - security-override (checked in in cr_security.py)
+        # - security-policy (checked in in cr_security.py)
         self.required_keys = ['start', 'description']
-        self.optional_keys = ['stop', 'poststop', 'stop-timeout'] + \
-            self.snappy_exe_security
+        self.optional_keys = ['stop',
+                              'poststop',
+                              'stop-timeout',
+                              'bus-name',
+                              'ports'
+                              ] + self.snappy_exe_security
 
         self.systemd_files = dict()  # click-show-files and tests
         self.systemd = dict()
 
         if self.is_snap and 'services' in self.pkg_yaml:
+            if len(self.pkg_yaml['services']) == 0:
+                error("package.yaml malformed: 'services' is empty")
             for service in self.pkg_yaml['services']:
                 if 'name' not in service:
                     error("package.yaml malformed: required 'name' not found "
@@ -156,6 +165,10 @@ class ClickReviewSystemd(ClickReview):
                                 (my_dict[app][o], o)
                         else:
                             found = True
+                    elif o == 'ports':
+                        if not isinstance(my_dict[app][o], dict):
+                            t = 'error'
+                            s = "'%s' is not dictionary" % o
                     elif not isinstance(my_dict[app][o], str):
                         t = 'error'
                         s = "'%s' is not a string" % o
@@ -338,13 +351,193 @@ class ClickReviewSystemd(ClickReview):
             self._add_result(t, n, s)
 
     def check_service_stop_timeout(self):
-        '''Check snappy-systemd'''
+        '''Check snappy-systemd stop-timeout'''
         self._verify_service_stop_timeout(self.systemd, 'hook')
 
     def check_snappy_service_stop_timeout(self):
-        '''Check snappy package.yaml top-timeout'''
+        '''Check snappy package.yaml stop-timeout'''
         if not self.is_snap or 'services' not in self.pkg_yaml:
             return
         self._verify_service_stop_timeout(self._create_dict(
                                           self.pkg_yaml['services']),
                                           'package_yaml')
+
+    def _verify_service_bus_name(self, pkgname, my_dict, test_str):
+        for app in sorted(my_dict):
+            if 'bus-name' not in my_dict[app]:
+                continue
+            f = os.path.basename(self.systemd_files[app])
+
+            t = 'info'
+            n = '%s_bus-name_empty_%s' % (test_str, f)
+            s = 'OK'
+            if len(my_dict[app]['bus-name']) == 0:
+                t = 'error'
+                s = "'bus-name' is empty"
+                self._add_result(t, n, s)
+                continue
+            self._add_result(t, n, s)
+
+            t = 'info'
+            n = '%s_bus-name_format_%s' % (test_str, f)
+            l = None
+            s = 'OK'
+            if not re.search(r'^[A-Za-z0-9][A-Za-z0-9_-]*(\.[A-Za-z0-9][A-Za-z0-9_-]*)+$',
+                             my_dict[app]['bus-name']):
+                t = 'error'
+                l = 'http://dbus.freedesktop.org/doc/dbus-specification.html'
+                s = "'%s' is not of form '^[A-Za-z0-9][A-Za-z0-9_-]*(\\.[A-Za-z0-9][A-Za-z0-9_-]*)+$'" % \
+                    (my_dict[app]['bus-name'])
+            self._add_result(t, n, s, l)
+
+            t = 'info'
+            n = '%s_bus-name_matches_name_%s' % (test_str, f)
+            s = 'OK'
+            suggested = [pkgname,
+                         "%s.%s" % (pkgname, app)
+                         ]
+            if self.is_snap and 'vendor' in self.pkg_yaml:
+                tmp = self.pkg_yaml['vendor'].split('@')
+                if len(tmp) > 1:
+                    rev = tmp[1].rstrip('>').split('.')
+                    rev.reverse()
+                    suggested.append("%s.%s" % (".".join(rev),
+                                                pkgname))
+                    suggested.append("%s.%s.%s" % (".".join(rev),
+                                                   pkgname,
+                                                   app))
+            found = False
+            for name in suggested:
+                if my_dict[app]['bus-name'].endswith(name):
+                    found = True
+                    break
+            if not found:
+                t = 'error'
+                s = "'%s' doesn't end with one of: %s" % \
+                    (my_dict[app]['bus-name'], ", ".join(suggested))
+            self._add_result(t, n, s)
+
+    def check_service_bus_name(self):
+        '''Check snappy-systemd bus-name'''
+        self._verify_service_bus_name(self.click_pkgname, self.systemd, 'hook')
+
+    def check_snappy_service_bus_name(self):
+        '''Check snappy package.yaml bus-name'''
+        if not self.is_snap or 'services' not in self.pkg_yaml:
+            return
+        self._verify_service_bus_name(self.pkg_yaml['name'],
+                                      self._create_dict(
+                                      self.pkg_yaml['services']),
+                                      'package_yaml')
+
+    def _verify_service_ports(self, pkgname, my_dict, test_str):
+        for app in sorted(my_dict):
+            if 'ports' not in my_dict[app]:
+                continue
+            f = os.path.basename(self.systemd_files[app])
+
+            t = 'info'
+            n = '%s_ports_empty_%s' % (test_str, f)
+            s = 'OK'
+            if len(my_dict[app]['ports'].keys()) == 0:
+                t = 'error'
+                s = "'ports' must contain 'internal' and/or 'external'"
+                self._add_result(t, n, s)
+                continue
+            self._add_result(t, n, s)
+
+            t = 'info'
+            n = '%s_ports_bad_key_%s' % (test_str, f)
+            s = 'OK'
+            badkeys = []
+            for i in my_dict[app]['ports'].keys():
+                if i not in ['internal', 'external']:
+                    badkeys.append(i)
+            if len(badkeys) > 0:
+                t = 'error'
+                s = "Unknown '%s' found in 'ports'" % ",".join(badkeys)
+            self._add_result(t, n, s)
+
+            port_pat = re.compile(r'^[0-9]+/[a-z0-9\-]+$')
+            for key in ['internal', 'external']:
+                if key not in my_dict[app]['ports']:
+                    continue
+
+                if len(my_dict[app]['ports'][key].keys()) < 1:
+                    t = 'error'
+                    n = '%s_ports_%s_%s' % (test_str, key, f)
+                    s = 'Could not find any %s ports' % key
+                    self._add_result(t, n, s)
+                    continue
+
+                for tagname in my_dict[app]['ports'][key]:
+                    entry = my_dict[app]['ports'][key][tagname]
+                    if len(entry.keys()) < 1:
+                        t = 'error'
+                        n = '%s_ports_%s_%s' % (test_str, key, f)
+                        s = 'Could not find any subkeys for %s' % tagname
+                        self._add_result(t, n, s)
+                        continue
+                    # Annoyingly, the snappy-systemd file uses 'Port' and
+                    # 'Negotiable' instead of 'port' and 'negotiable' from the
+                    # yaml
+                    if (test_str == 'package_yaml' and
+                            'negotiable' not in entry and
+                            'port' not in entry) or \
+                       (test_str == 'hook' and
+                            'Negotiable' not in entry and
+                            'Port' not in entry):
+                        t = 'error'
+                        n = '%s_ports_%s_invalid_%s' % (test_str, key, f)
+                        s = "Must specify specify at least 'port' or " + \
+                            "'negotiable'"
+                        self._add_result(t, n, s)
+                        continue
+
+                    # port
+                    subkey = 'port'
+                    if test_str == 'hook':
+                        subkey = 'Port'
+                    t = 'info'
+                    n = '%s_ports_%s_%s_format' % (test_str, tagname, subkey)
+                    s = 'OK'
+                    if subkey not in entry:
+                        s = 'OK (skipped, not found)'
+                    else:
+                        tmp = entry[subkey].split('/')
+                        if not port_pat.search(entry[subkey]) or \
+                           int(tmp[0]) < 1 or int(tmp[0]) > 65535:
+                            t = 'error'
+                            s = "'%s' should be of form " % entry[subkey] + \
+                                "'port/protocol' where port is an integer " + \
+                                "(1-65535) and protocol is found in " + \
+                                "/etc/protocols"
+                    self._add_result(t, n, s)
+
+                    # negotiable
+                    subkey = 'negotiable'
+                    if test_str == 'hook':
+                        subkey = 'Negotiable'
+                    t = 'info'
+                    n = '%s_ports_%s_%s_format' % (test_str, tagname, subkey)
+                    s = 'OK'
+                    if subkey not in entry:
+                        s = 'OK (skipped, not found)'
+                    elif entry[subkey] not in [True, False]:
+                        t = 'error'
+                        s = "'%s: %s' should be either 'yes' or 'no'" % \
+                            (subkey, entry[subkey])
+                    self._add_result(t, n, s)
+
+    def check_service_ports(self):
+        '''Check snappy-systemd ports'''
+        self._verify_service_ports(self.click_pkgname, self.systemd, 'hook')
+
+    def check_snappy_service_ports(self):
+        '''Check snappy package.yaml ports'''
+        if not self.is_snap or 'services' not in self.pkg_yaml:
+            return
+        self._verify_service_ports(self.pkg_yaml['name'],
+                                   self._create_dict(
+                                   self.pkg_yaml['services']),
+                                   'package_yaml')
