@@ -1899,3 +1899,101 @@ class TestClickReviewSecurity(cr_tests.TestClickReview):
         report = c.click_report
         expected_counts = {'info': None, 'warn': 0, 'error': 1}
         self.check_results(report, expected_counts)
+
+    def test_check_template_online_account_provider(self):
+        '''Test check_template_online_account_provider'''
+        self.set_test_account(self.default_appname, "account-provider", "foo")
+        self.set_test_security_manifest(self.default_appname,
+                                        "template", "ubuntu-account-plugin")
+        self.set_test_security_manifest(self.default_appname,
+                                        "policy_groups", ["accounts"])
+        c = ClickReviewSecurity(self.test_name)
+        c.check_template_online_accounts_provider()
+        report = c.click_report
+        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
+        self.check_results(report, expected_counts)
+
+    def test_check_template_online_account_provider_no_hook(self):
+        '''Test check_template_online_account_provider'''
+        self.set_test_security_manifest(self.default_appname,
+                                        "template", "ubuntu-account-plugin")
+        c = ClickReviewSecurity(self.test_name)
+        c.check_template_online_accounts_provider()
+        report = c.click_report
+        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
+        self.check_results(report, expected_counts)
+
+    def test_check_template_online_account_provider_wrong_template(self):
+        '''Test check_template_online_account_provider - wrong template'''
+        self.set_test_account(self.default_appname, "account-provider", "foo")
+        self.set_test_security_manifest(self.default_appname,
+                                        "template", "ubuntu-webapp")
+        self.set_test_security_manifest(self.default_appname,
+                                        "policy_groups", ["accounts"])
+        c = ClickReviewSecurity(self.test_name)
+        c.check_template_online_accounts_provider()
+        report = c.click_report
+        expected_counts = {'info': None, 'warn': 0, 'error': 1}
+        self.check_results(report, expected_counts)
+
+    def test_check_template_online_account_provider_wrong_template2(self):
+        '''Test check_template_online_account_provider - default template'''
+        self.set_test_account(self.default_appname, "account-provider", "foo")
+        self.set_test_security_manifest(self.default_appname,
+                                        "template", None)
+        self.set_test_security_manifest(self.default_appname,
+                                        "policy_groups", ["accounts"])
+        c = ClickReviewSecurity(self.test_name)
+        c.check_template_online_accounts_provider()
+        report = c.click_report
+        expected_counts = {'info': None, 'warn': 0, 'error': 1}
+        self.check_results(report, expected_counts)
+
+    def test_check_template_online_account_qml_plugin(self):
+        '''Test check_template_online_account_qml_plugin'''
+        self.set_test_account(self.default_appname, "account-qml-plugin", "foo")
+        self.set_test_security_manifest(self.default_appname,
+                                        "template", "ubuntu-account-plugin")
+        self.set_test_security_manifest(self.default_appname,
+                                        "policy_groups", ["accounts"])
+        c = ClickReviewSecurity(self.test_name)
+        c.check_template_online_accounts_qml_plugin()
+        report = c.click_report
+        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
+        self.check_results(report, expected_counts)
+
+    def test_check_template_online_account_qml_plugin_no_hook(self):
+        '''Test check_template_online_account_qml_plugin'''
+        self.set_test_security_manifest(self.default_appname,
+                                        "template", "ubuntu-account-plugin")
+        c = ClickReviewSecurity(self.test_name)
+        c.check_template_online_accounts_qml_plugin()
+        report = c.click_report
+        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
+        self.check_results(report, expected_counts)
+
+    def test_check_template_online_account_qml_plugin_wrong_template(self):
+        '''Test check_template_online_account_qml_plugin - wrong template'''
+        self.set_test_account(self.default_appname, "account-qml-plugin", "foo")
+        self.set_test_security_manifest(self.default_appname,
+                                        "template", "ubuntu-webapp")
+        self.set_test_security_manifest(self.default_appname,
+                                        "policy_groups", ["accounts"])
+        c = ClickReviewSecurity(self.test_name)
+        c.check_template_online_accounts_qml_plugin()
+        report = c.click_report
+        expected_counts = {'info': None, 'warn': 0, 'error': 1}
+        self.check_results(report, expected_counts)
+
+    def test_check_template_online_account_qml_plugin_wrong_template2(self):
+        '''Test check_template_online_account_qml_plugin - default template'''
+        self.set_test_account(self.default_appname, "account-qml-plugin", "foo")
+        self.set_test_security_manifest(self.default_appname,
+                                        "template", None)
+        self.set_test_security_manifest(self.default_appname,
+                                        "policy_groups", ["accounts"])
+        c = ClickReviewSecurity(self.test_name)
+        c.check_template_online_accounts_qml_plugin()
+        report = c.click_report
+        expected_counts = {'info': None, 'warn': 0, 'error': 1}
+        self.check_results(report, expected_counts)
